import Navigine


class Convert: NSObject {
    class func deviceInfo(toObject deviceInfo: NCDeviceInfo?) -> Dictionary<AnyHashable, AnyHashable> {
        var data: Dictionary<AnyHashable, AnyHashable> = [:]
        data["x"] = deviceInfo?.x
        data["y"] = deviceInfo?.y
        data["id"] = deviceInfo?.id
        data["r"] = self.float(toNSNumber: deviceInfo?.r ?? 0.0)
        data["azimuth"] = self.float(toNSNumber: deviceInfo?.azimuth ?? 0.0)
        data["location"] = self.int(toNSNumber: deviceInfo?.location ?? 0)
        data["subLocation"] = self.int(toNSNumber: deviceInfo?.sublocation ?? 0)

        var paths: [AnyHashable] = []
        for i in 0..<(deviceInfo?.paths?.count ?? 0) {
            let path = deviceInfo?.paths?[i]
            let path1 = self.path(toDict: path)
            paths.append(path1)
        }
        data["paths"] = paths

        var zones: [AnyHashable] = []
        for i in 0..<(deviceInfo?.zones?.count ?? 0) {
            let zone = deviceInfo?.zones?[i]
            let zone1 = self.zone(toDict: zone)
            zones.append(zone1)
        }
        data["zones"] = zones

        return data
    }

    class func path(toDict path: NCRoutePath?) -> Dictionary<AnyHashable, AnyHashable> {
        var pathDict: Dictionary<AnyHashable, AnyHashable> = [:]
        pathDict["length"] = path?.lenght
        
        var points: [AnyHashable] = []
        for i in 0..<(path?.points.count ?? 0){
            let point = path?.points[i]
            let point1 = self.locationPoint(toDict: point)
            points.append(point1)
        }
        
        pathDict["points"] = points
        
        var events: [AnyHashable] = []
        for i in 0..<(path?.events.count ?? 0){
            let event = path?.events[i]
            let event1 = self.event(toDict: event)
            events.append(event1)
        }
        
        pathDict["events"] = events

        return pathDict
    }

    class func zone(toDict zone: NCZone?) -> Dictionary<AnyHashable, AnyHashable> {
        var zoneDict: Dictionary<AnyHashable, AnyHashable> = [:]
        
        zoneDict["alias"] = zone?.alias
        zoneDict["color"] = zone?.color
        zoneDict["id"] = zone?.id
        zoneDict["location"] = zone?.location
        zoneDict["name"] = zone?.name
        zoneDict["subLocation"] = zone?.sublocation
        
        var points: [AnyHashable] = []
        for i in 0..<(zone?.points.count ?? 0){
            let point = zone?.points[i]
            let point1 = self.locationPoint(toDict: point)
            points.append(point1)
        }
        
        zoneDict["points"] = points

        return zoneDict
    }
    class func event(toDict event: NCRouteEvent?) -> Dictionary<AnyHashable, AnyHashable> {
        var eventDict: Dictionary<AnyHashable, AnyHashable> = [:]
        eventDict["length"] = event?.value
        eventDict["type"] = event?.type
        eventDict["distance"] = event?.distance
        return eventDict
    }
    
    class func locationPoint(toDict locationPoint: NCLocationPoint?) -> Dictionary<AnyHashable, AnyHashable> {
        var locationPointDict: Dictionary<AnyHashable, AnyHashable> = [:]
        
        locationPointDict["location"] = locationPoint?.location
        locationPointDict["subLocation"] = locationPoint?.sublocation
        locationPointDict["x"] = locationPoint?.x
        locationPointDict["y"] = locationPoint?.y

        return locationPointDict
    }
    
    class func venue(toDict venue: NCVenue?) -> Dictionary<AnyHashable, AnyHashable> {
        var venueDict: Dictionary<AnyHashable, AnyHashable> = [:]
        
        venueDict["id"] = venue?.id
        venueDict["location"] = venue?.location
        venueDict["subLocation"] = venue?.sublocation
        venueDict["x"] = venue?.x
        venueDict["y"] = venue?.y
        venueDict["name"] = venue?.name
        venueDict["alias"] = venue?.alias
        venueDict["phone"] = venue?.phone
        venueDict["image"] = venue?.image
        venueDict["description"] = venue?.description

        return venueDict
    }
    
    class func beacon(toDict beacon: NCBeacon?) -> Dictionary<AnyHashable, AnyHashable> {
        var beaconDict: Dictionary<AnyHashable, AnyHashable> = [:]
        
        beaconDict["id"] = beacon?.id
        beaconDict["location"] = beacon?.location
        beaconDict["subLocation"] = beacon?.sublocation
        beaconDict["major"] = beacon?.major
        beaconDict["minor"] = beacon?.minor
        beaconDict["uuid"] = beacon?.uuid
        beaconDict["name"] = beacon?.name
        beaconDict["x"] = beacon?.x
        beaconDict["y"] = beacon?.y
        return beaconDict
    }
    
    class func locationPoint(fromDict locationPoint: Dictionary<String, Any>) -> NCLocationPoint {
        return NCLocationPoint.init(location: locationPoint["location"] as! Int, sublocation: locationPoint["subLocation"] as! Int, x: locationPoint["x"] as! NSNumber, y: locationPoint["y"] as! NSNumber)
    }
    
    class func subLocation(toDict subLocation:NCSublocation?) -> Dictionary<AnyHashable, AnyHashable> {
        var data: Dictionary<AnyHashable, AnyHashable> = [:]
        
        
        data["id"] = subLocation?.id
        data["location"] = subLocation?.location
        data["name"] = subLocation?.name
        data["svgFile"] = subLocation?.svgFile
        data["pngFile"] = subLocation?.pngFile
        data["jpgFile"] = subLocation?.jpgFile
        data["width"] = subLocation?.width
        data["height"] = subLocation?.height
        data["azimuth"] = subLocation?.azimuth
        data["latitude"] = subLocation?.coordinates?.latitude
        data["longitude"] = subLocation?.coordinates?.longitude
        data["image"] = subLocation?.sublocationImage.imageData
        
        var zones: [AnyHashable] = []
        for i in 0..<(subLocation?.zones.count ?? 0){
            let zone = subLocation?.zones[i]
            let zone1 = self.zone(toDict: zone)
            zones.append(zone1)
        }
        
        data["zones"] = zones
        
        var venues: [AnyHashable] = []
        for i in 0..<(subLocation?.venues.count ?? 0){
            let venue = subLocation?.venues[i]
            let venue1 = self.venue(toDict: venue)
            venues.append(venue1)

        }
        
        data["venues"] = venues
        
        
        var beacons: [AnyHashable] = []
        for i in 0..<(subLocation?.beacons.count ?? 0){
            let beacon = subLocation?.beacons[i]
            let beacon1 = self.beacon(toDict: beacon)
            beacons.append(beacon1)
        }
        
        data["beacons"] = beacons

        return data
    }

    class func locationInfo(toObject location: NCLocation?) -> Dictionary<AnyHashable, AnyHashable> {
        var data: Dictionary<AnyHashable, AnyHashable> = [:]
        
        
        data["id"] = location?.id
        data["name"] = location?.name
        data["description"] = location?.description
        data["version"] = location?.version
        var subLocations: [Dictionary<AnyHashable, AnyHashable>] = []
        for i in 0..<(location?.sublocations.count ?? 0){
            let subLocation = location?.sublocations[i]
            let subLocation1 = self.subLocation(toDict: subLocation)
            subLocations.append(subLocation1)
        }
        
        data["subLocations"] = subLocations
        
        return data
    }

    class func float(toNSNumber f: Float) -> NSNumber {
        return NSNumber(value: f)
    }

    class func int(toNSNumber value: Int) -> NSNumber {
        return NSNumber(value: value)
    }
}
